#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Mesh Maker
#      By:	sdmitch
#   Usage:	Creat a regular trianglated network of a given cell size of a selected face
#	 Note:	Made cell size a global variable, modified for tilted faces Jun 2011
#    Date:	Jun 2011
#------------------------------------------------------------------------------------------------

module Mesh_Gen

	def self.mesh_maker
		mod=Sketchup.active_model
		ent=mod.entities
		sel=mod.selection
		$Mesh_CS = 20.cm if !$Mesh_CS
		if sel.first.is_a? Sketchup::Face
			ans=UI.inputbox(["Cell Size:"],[$Mesh_CS],"Mesh Generator")
			if !ans then return end
			mod.start_operation "Pattern Generator"
			$Mesh_CS=ans[0].to_l; face=sel.first; edges=[]; edges=face.edges
			pts=[]; pts=face.outer_loop.vertices.collect {|v| v.position}
			pl=face.plane;ctr=face.bounds.center; vec=face.normal; vec.reverse! if vec.z < 0; 
			ctr=ctr.project_to_plane pl if !ctr.on_plane? pl
			maxd=0.0;pts.each {|p| d=ctr.distance(p); maxd=d if maxd<d}
			xa,ya,za=vec.axes; pt0=ctr.offset(xa.reverse,maxd).offset(ya.reverse,maxd)
			dx=maxd*2;dy=maxd*2; nx=(dx/$Mesh_CS).ceil; ny=(dy/$Mesh_CS).ceil; data=[]; row=[]
			# Generate the grid pattern
			for i in 0..ny
				row[0] = pt0
				for j in 1..nx
					row[j]=row[j-1].offset(xa,$Mesh_CS)
					# ent.add_cpoint(row[j-1])
				end
				pt0=pt0.offset(ya,$Mesh_CS)
				data[i]=row; row=[]
			end
			grp = ent.add_group; @ge = grp.entities;    # create a group for c_line entities
			# Draw the horizontal lines
			for i in 0..ny
				for j in 1..nx
					start_pt_outside = (face.classify_point(data[i][j-1]) == Sketchup::Face::PointOutside)
					end_pt_outside = (face.classify_point(data[i][j]) == Sketchup::Face::PointOutside)
					if !(start_pt_outside && end_pt_outside)
						if !(start_pt_outside || end_pt_outside)
							@ge.add_cline(data[i][j-1],data[i][j])
						else
							poi=self.find_intersection(data[i][j-1],data[i][j],edges)
							if poi
								@ge.add_cline(poi,data[i][j]) if start_pt_outside
								@ge.add_cline(data[i][j-1],poi) if end_pt_outside
							end
						end
					end
				end
			end
			# Draw the vertical lines
			for j in 0..nx
				for i in 1..ny
					start_pt_outside = (face.classify_point(data[i-1][j]) == Sketchup::Face::PointOutside)
					end_pt_outside = (face.classify_point(data[i][j]) == Sketchup::Face::PointOutside)
					if !(start_pt_outside && end_pt_outside)
						if !(start_pt_outside || end_pt_outside)
							@ge.add_cline(data[i-1][j],data[i][j])
						else
							poi=self.find_intersection(data[i-1][j],data[i][j],edges)
							if poi
								@ge.add_cline(poi,data[i][j]) if start_pt_outside
								@ge.add_cline(data[i-1][j],poi) if end_pt_outside
							end
						end
					end
				end
			end
			# Draw the diagonal lines
			for i in 1..ny
				for j in 1..nx
					start_pt_outside = (face.classify_point(data[i][j-1]) == Sketchup::Face::PointOutside)
					end_pt_outside = (face.classify_point(data[i-1][j]) == Sketchup::Face::PointOutside)
					if !(start_pt_outside && end_pt_outside)
						if (start_pt_outside || end_pt_outside)
							poi=self.find_intersection(data[i][j-1],data[i-1][j],edges)
							if poi
								@ge.add_cline(poi,data[i-1][j]) if start_pt_outside
								@ge.add_cline(data[i][j-1],poi) if end_pt_outside
							end
						else
							@ge.add_cline(data[i][j-1],data[i-1][j])
						end
					end
				end
			end
	#		convert construction lines to soft and smooth edges and erase guide lines	
			@ge.each {|e| edg=ent.add_line(e.start,e.end);edg.smooth=true;edg.soft=true}
			ent.erase_entities grp
			mod.commit_operation
		else
			UI.messagebox "No face selected"
		end
	end
	
	def self.find_intersection(p0,p1,edges)
		ln0=[p0,p0.vector_to(p1)]
		for e in edges
			ln1=e.line; pp=Geom.intersect_line_line(ln0,ln1)
			if pp
				if self.is_between(pp,e.start.position,e.end.position)
					if self.is_between(pp,p0,p1)
						return pp
					end
				end
			end
		end
		return nil
	end
	
	def  self.is_between(p0,p1,p2)
	# --------------------------------------------------------------------------------
	#	Determine if point p0 is on the line and between end points p1 and p2
	# --------------------------------------------------------------------------------
		d1=p0.distance p1 
		d2=p0.distance p2 
		d3=p1.distance p2 
		return true if ([d1,d2].max <= d3)
		return false # p0 is not on line or between p1 and p2
	end
end

filename=File.basename(__FILE__)

if !file_loaded?(filename)
	UI.menu("Plugins").add_item("Mesh Maker") { Mesh_Gen.mesh_maker }
end

file_loaded(filename)

